'''
!/usr/bin/env python
coding: utf-8

DESCRIPTION:
Creates/updates a dictionary a PDF's annotations for use in Notes app.

INPUTS:
PDF file location, meta data. Change to source class object.
    - metadata
    - annotations
    - dummy vars (read)
    - functions
        - find_annots
        - update_annots
        - xxx

OUTPUTS:
Dictionary with PDF metadata and annotations
'''

# IMPORTS
import sys
import os
import fitz


def _check_contain(rect_annot, rect_reference, threshold=0.75):
    '''Check if word rectangle overlaps with annotation rectangle'''
    x_a1, y_a1, x_a2, y_a2 = rect_annot
    x_b1, y_b1, x_b2, y_b2 = rect_reference

    if x_a1 >= x_b2 or x_b1 >= x_a2:
        return False
    elif y_a1 >= y_b2 or y_b1 >= y_a2:
        return False
    else:
        b_area = (y_b2 - y_b1) * (x_b2 - x_b1)
        overlap_area = (
                        (min(y_a2, y_b2) - max(y_a1, y_b1))
                        * (min(x_a2, x_b2) - max(x_a1, x_b1))
                       )
        return (overlap_area / b_area) > threshold


def _iterate_words(page):
    '''Iterate through all words in a page and return word'''
    for wb in sorted(page.get_text('words'), key=lambda w: (w[1], w[0])):
        yield(wb)


def _get_highlight_text(annot):
    '''Get highlighted text'''
    annot_text_raw = ''
    rect_counts = len(annot.vertices) // 4
    for i in range(rect_counts):
        for word in _iterate_words(annot.parent):
            if _check_contain(
                              annot.vertices[i * 4]
                              + annot.vertices[(i * 4) + 3],
                              word[:4],
                             ):
                annot_text_raw = annot_text_raw + ' ' + word[4]
    return annot_text_raw


def _create_annot(annot, source):
    '''Create annot entry in source_entry dict
       for sticky comments and highlights
    '''
    # Get text from sticky comment
    if(annot.type[0] == 0):
        annot_text_raw = annot.info['content']

    # Get text from highlight
    elif(annot.type[0] == 8):
        annot_text_raw = _get_highlight_text(annot)

    else:
        annot_text_raw = 'None'

    # Create annot entry
    annot_text = ('PAGE '
                  + str(annot.parent.number + 1)
                  + ' ('
                  + annot.type[1]
                  + '): '
                  + annot_text_raw
                  )
    annot_entry = {
                       'page': annot.parent.number + 1,
                       'type': annot.type[1],
                       'text': annot_text
                       }

    # Append annot entry if not already present
    if annot_entry not in source['annots']:
        source['annots'].append(annot_entry)
        source['all_notes'] += '\n\n' + annot_text 
        print('\t\t\tAnnot added to dictionary')
    else:
        print('\t\t\tAnnot already in dictionary')


def _main(source):
    attachment = source['attachment']

    if attachment == None:
        return source
    try:
        file_path = os.path.normpath(attachment)
        doc = fitz.open(file_path)
        print('\t\tExtracting annotations')

        for page in doc.pages():
            for annot in page.annots():
                _create_annot(annot, source)
        return source
   
    except RuntimeError:
        print('\t\tUnable to extract annotations')
        attachment = None 
        return source

if __name__ == '__main__':
    _main(sys.argv[1])



import fitz
import sys
import os
import pdfkit
import fitz
import tkinter as tk

import zoresearch.gui

class Source:
	def __init__(self, metadata):

		# Set initial values
		self.title= ''
		self.short_title = ''
		self.url = ''
		self.attachment = ''
		self.date_added = ''
		self.collections = []
		self.annots = []
		self.all_notes = ''
		self.read = False
		self.starred = False

		# Add any passed data as attributes
		for key in metadata:
		 	setattr(self, key, metadata[key])


	def _describe(self):
		print(f'SOURCE DESCRIPTION\nTitle: {self.title}\nKey: {self.key}\nURL: {self.url}\nAll notes: {self.all_notes}')


	def _create_label(self, frame):
		
		if self.short_title is not None:
			text = self.short_title

		elif self.title is not None:
			text = self.title

		else:
			text = 'Title unavailable'

		if self.starred:
			text =u'\u2605 ' +  text

		if len(text) > 80:
			text = text[ :80] + '...'

		self.label = tk.Label(
						frame, 
						text=text,
						width=30,
						font=('open sans',10),
                        wraplength=200, 
                        bg='#E5E5E5', 
                        justify=tk.LEFT,
                        anchor='w',
                        pady=10,
                        cursor='hand2'
						)
		if self.read == False:
			self.label.config(font=('open sans', 10, 'bold'))

		self.label.bind('<Enter>', zoresearch.gui._hover)
		self.label.bind('<Leave>', zoresearch.gui._leave)
		

	def _html_to_pdf(self, html):
		# print('\t\tConverting HTML file to PDF: {}'.format(html))
		html = self.attachment + html
		pdf_path = html[ : -4] + 'pdf'
		try:
			self.attachment = pdfkit.from_file(html, pdf_path, verbose=False, options=(
					{
					'disable-javascript': True
					, 'load-error-handling': 'ignore'
					}
				)
			)
		
		except OSError:
			# print('\t\t\tWKHTML is complaining')
			pass
		
		self.attachment = pdf_path
		return


	def _get_attachment(self):
		# print('\t\tInitial attachment value: {}'.format(self.attachment))

		# Return if PDF is already located or if no attachment exists
		if self.attachment is None:
			return

		if self.attachment.endswith('.pdf'):
			# print('\t\tPDF already exists or no attachment: {}'.format(self.attachment))
			return

		# Otherwise, attachment points to directory. Convert HTML to PDF if needed
		else:
			# print('\t\t\tSearching for file in folder {}'.format(self.attachment))
			try:
				directory = os.listdir(self.attachment)
				pdf = [match for match in directory if '.pdf' in match]
				if pdf:
					# print('\t\tPDF found in source folder: {}'.format(pdf[0]))
					self.attachment = self.attachment + pdf[0]
					return

				else:
					html = [match for match in directory if '.html' in match]
					if html:
						self.attachment = self._html_to_pdf(html[0])
						return
					else:
						# print('\t\tNo attachment found in folder; return none')
						self.attachment = None
						return
			
			except FileNotFoundError:
				return


	def _check_contain(self, rect_annot, rect_reference, threshold=0.75):
	    '''Check if word rectangle overlaps with annotation rectangle'''
	    x_a1, y_a1, x_a2, y_a2 = rect_annot
	    x_b1, y_b1, x_b2, y_b2 = rect_reference

	    if x_a1 >= x_b2 or x_b1 >= x_a2:
	        return False
	    elif y_a1 >= y_b2 or y_b1 >= y_a2:
	        return False
	    else:
	        b_area = (y_b2 - y_b1) * (x_b2 - x_b1)
	        overlap_area = (
	                        (min(y_a2, y_b2) - max(y_a1, y_b1))
	                        * (min(x_a2, x_b2) - max(x_a1, x_b1))
	                       )
	        return (overlap_area / b_area) > threshold


	def _iterate_words(self, page):
	    '''Iterate through all words in a page and return word'''
	    for wb in sorted(page.get_text('words'), key=lambda w: (w[1], w[0])):
	        yield(wb)


	def _get_highlight_text(self, annot):
	    '''Get highlighted text'''
	    annot_text_raw = ''
	    rect_counts = len(annot.vertices) // 4
	    for i in range(rect_counts):
	        for word in self._iterate_words(annot.parent):
	            if self._check_contain(
	                              annot.vertices[i * 4]
	                              + annot.vertices[(i * 4) + 3],
	                              word[:4],
	                             ):
	                annot_text_raw = annot_text_raw + ' ' + word[4]
	    return annot_text_raw



	def _create_annot(self, annot):
	    '''Create annot entry in source_entry dict
	       for sticky comments and highlights
	    '''
	    # Get text from sticky comment
	    if(annot.type[0] == 0):
	        annot_text_raw = annot.info['content']

	    # Get text from highlight
	    elif(annot.type[0] == 8):
	        annot_text_raw = self._get_highlight_text(annot)

	    else:
	        annot_text_raw = 'None'

	    # Create annot entry
	    annot_text = ('PAGE '
	                  + str(annot.parent.number + 1)
	                  + ' ('
	                  + annot.type[1]
	                  + '): '
	                  + annot_text_raw
	                  )
	    annot_entry = {
	                       'page': annot.parent.number + 1,
	                       'type': annot.type[1],
	                       'text': annot_text
	                       }

	    # Append annot entry if not already present
	    if annot_entry not in self.annots:
	        self.annots.append(annot_entry)
	        self.all_notes += '\n\n' + annot_text 
	        # print('\t\t\tAnnot added to dictionary')
	    else:
	        # print('\t\t\tAnnot already in dictionary')
	        pass



	def _get_annots(self):

	    if self.attachment == None:
	        return
	    try:
	        file_path = os.path.normpath(self.attachment)
	        doc = fitz.open(file_path)
	        # print('\t\tExtracting annotations')

	        for page in doc.pages():
	            for annot in page.annots():
	                self._create_annot(annot)
	        return
	   
	    except RuntimeError:
	        # print('\t\tUnable to extract annotations')
	        self.attachment = None 
	        return


# Import external modules
import sys
import os
from tendo import singleton

# Import internal modules
import zoresearch.sources_db
import zoresearch.gui



def open(zotero_folder, collection_name='all'):
	# Terminate program if other instance is running
	me = singleton.SingleInstance() 

	# Get queried collection name from user entry
	# if len(collection_name.split()) > 1:
	# 	collection_name = ' '.join(str(entry.lower()) for entry in collection_name) 
	collection_name = collection_name.lower()
	# collection_name = collection_name.replace('"', '') # Remove any quotation marks
	print(f'OPENING COLLECTION: {collection_name.title()}')

	# Query Zotero database and return sources
	zotero_folder = zotero_folder.replace('"', '') # Remove any quotation marks
	zotero_folder = zotero_folder.replace("'", "") # Remove any quotation marks

	zotero_folder = os.path.normpath(zotero_folder)
	zotero_location = os.path.normpath(zotero_folder + '\\zotero.sqlite')
	zotero_data_raw = zoresearch.sources_db._sql_query(zotero_location)
	zotero_data_processed = zoresearch.sources_db._process_data(zotero_data_raw, zotero_folder)

	# Add/remove sources from local DB to match Zotero data
	app_data_location = os.path.normpath(zotero_folder + '\\app_data.json')
	app_data = zoresearch.sources_db._update_data(app_data_location, zotero_data_processed)

	# Start GUI
	zoresearch.gui._main(app_data, collection_name, app_data_location)


if __name__ == '__main__':
	print("This is my file to test Python's execution methods.")
	print("The variable __name__ tells me which context this file is running in.")
	print("The value of __name__ is:", repr(__name__))
	zotero_folder = sys.argv[1]
	collection_name = ' '.join(sys.argv[2:]).strip()
	if not collection_name:
		collection_name = 'all'
	print(zotero_folder)
	print(collection_name)
	open(zotero_folder, collection_name)



# from tkinter import *
from tkinter import Tk, font
import tkinter as tk
import webbrowser
import fitz
import json
import time
import os
from importlib import resources


import zoresearch.sources_db


class Note_gui:
	def __init__(self, app_data, collection_name, app_data_location):
		
		# Create Tkinter window
		self.root = tk.Tk()
		self.root.title("Research Reader: " + collection_name.title() + " Collection") 
		self.root.geometry("1200x450")
		self.root.config(bg='#E5E5E5')
		icon_path = self.get_icon()
		self.root.iconbitmap(icon_path)
		print('CURRENT DIR:' + os.getcwd())

		# Initialize metadata
		self.app_data_location = app_data_location
		self.app_data = app_data
		self.collection_name = collection_name
		self.show_pdf = False

		# Initialize collection dropdown
		self.collection_names = sorted(set(collection.title() for source in self.app_data for collection in source.collections))
		self.dropdown_selection = tk.StringVar()
		self.dropdown_selection.set(self.collection_name.title())

		# Initialize source type dropdown
		self.source_types = ['All', 'Starred', 'Unread']
		self.source_type_dropdown_selection = tk.StringVar()
		self.source_type_dropdown_selection.set('All')

		# Create main display and widgets
		self._main_display()
		for source in self.app_data:
			source._create_label(self.frame)

		# Display queried collection
		self._display_collection()

		# Display first source in collection
		self.selected_source = self.selected_collection[0]
		self._display_source(self.selected_source)

		# Configure GUI
		# self.root.rowconfigure(0, minsize=50)
		self.root.rowconfigure(3, weight=3)
		self.root.columnconfigure(2, weight=0, minsize=50)
		self.root.columnconfigure(3, weight=0, minsize=50)
		self.root.columnconfigure(4, weight=3, minsize=200)


		# Run GUI
		self.root.bind('<Control-w>', lambda e: self._on_closing())
		self.root.protocol("WM_DELETE_WINDOW", lambda: self._on_closing())
		self.root.mainloop()


	def get_icon(self):
		with resources.path("zoresearch.icon", "zoresearch.ico") as f:
			data_file_path = f
		return data_file_path


	def _on_closing(self):
		print('Clicked close')
		self._save_source()
		self._save_data()
		self.root.destroy()


	def _save_source(self):
		self.selected_source.all_notes = self.text_widget.get("1.0",tk.END)


	def _save_data(self):
		data = [vars(source) for source in self.app_data]
		with open(self.app_data_location, 'w') as f:
			json.dump(data, f, indent=4, default=str)
		print("Saved project file")


	def _on_frame_configure(self):
		'''Reset the scroll region to encompass the inner frame'''
		self.canvas.configure(scrollregion=self.canvas.bbox("all"))


	def _on_mousewheel(self, event):
		change = int(-1*(event.delta/120))
		self.canvas.yview_scroll(change, "units")


	def _set_mousewheel(self, command):
	    """Activate / deactivate mousewheel scrolling when 
	    cursor is over / not over the widget respectively."""
	    self.canvas.bind("<Enter>", lambda _: self.canvas.bind_all('<MouseWheel>', command))
	    self.canvas.bind("<Leave>", lambda _: self.canvas.unbind_all('<MouseWheel>'))


	def _open_link(self, dest):
		if dest == '':
			return
		else:
			webbrowser.open_new(dest)	


	def _mark_unread(self, e):

		# Change read status of selected source
		self.selected_source.read = False

		# Change source label
		self.selected_source.label.configure(font=('open sans', 10, 'bold'))


		# Recenter scroll bar on selected source
		# source_position = float((self.selected_collection.index(self.selected_source)) / (len(self.selected_collection)))
		# self.canvas.yview_moveto(source_position)


	def _star_source(self, e):
		self.selected_source.starred = not self.selected_source.starred

		# Starred source
		if self.selected_source.starred == True:
			text = u'\u2605 ' + self.selected_source.label.cget('text')
			self.selected_source.label.configure(text=text)
			self.star_label.config(text = u'\u2605')

		# Unstarred source
		else:
			# Change source label
			self.star_label.config(text = u'\u2606')
			text = self.selected_source.label.cget('text')[2:]
			self.selected_source.label.configure(text=text)



	def _select_collection(self):
		self.collection_name = self.dropdown_selection.get().lower()
		print(f'Change to {self.collection_name.title()} collection')

		self.root.title("Research Reader: " + self.collection_name.title() + " Collection") 
		self._save_source()
		self.text_widget.delete('1.0', tk.END)

		# Display queried collection
		self._display_collection()

		# Display first source in collection
		self.selected_source = self.selected_collection[0]
		self._display_source(self.selected_source)


	def _select_source_type(self):
		source_type = self.source_type_dropdown_selection.get().lower()
		print(source_type)
		self._save_source()
		self.text_widget.delete('1.0', tk.END)

		
		self._display_collection(source_type)
		self.selected_source = self.selected_collection[0]
		self._display_source(self.selected_source)


	def _main_display(self):

		# Create widgets
		self.canvas = tk.Canvas(self.root, background="#E5E5E5", width=220) 
		self.frame = tk.Frame(self.canvas, background='#E5E5E5')
		self.text_widget = tk.Text(self.root, bg='#F9F9F9', font=('open sans', 10), relief='flat')
		self.collections_dropdown = tk.OptionMenu(self.root, self.dropdown_selection, *self.collection_names, command = lambda _: self._select_collection())

		self.source_label = tk.Label(self.root, text='Sources', font=('open sans', 10), bg='#E5E5E5', padx=0)
		self.title_label = tk.Label(self.root, text='Title', font=('open sans', 10), bg='#E5E5E5', wraplength=700) 
		self.url_label = tk.Label(self.root, text='URL', font=('open sans', 10, 'italic'), fg='blue', bg='#E5E5E5', cursor='hand2',  wraplength=700)
		self.read_label = tk.Label(self.root, text='', font=('open sans', 10), bg='#E5E5E5', wraplength=50) 
		self.pdf_label = tk.Label(self.root, text='Show PDF', font=('open sans', 10), bg='#E5E5E5', wraplength=50)

		self.text_scroll = tk.Scrollbar(self.root, orient='vertical', command=self.text_widget.yview)
		self.source_scroll = tk.Scrollbar(self.root, orient="vertical", command=self.canvas.yview)

		self.pdf_frame = tk.Frame(self.root)
		self.pdf_scroll = tk.Scrollbar(self.pdf_frame, orient='vertical')
		self.pdf_text = tk.Text(self.pdf_frame, yscrollcommand=self.pdf_scroll.set, width=90, bg='#F9F9F9', font=('open sans', 10), relief='flat')

		self.star_label = tk.Label(self.root, text = u'\u2606', font=('open sans', 10), bg='#E5E5E5', padx=0)
		self.source_type_label = tk.OptionMenu(self.root, self.source_type_dropdown_selection, *self.source_types, command = lambda _: self._select_source_type())

		# Configure widgets
		self.canvas.create_window((0,0), window=self.frame, anchor="nw")
		self.canvas.configure(yscrollcommand=self.source_scroll.set)
		self.frame.bind("<Configure>", lambda event: self._on_frame_configure())
		self._set_mousewheel(lambda event: self._on_mousewheel(event))
		self.text_widget.configure(yscrollcommand=self.text_scroll.set)
		self.pdf_label.bind('<Button-1>', lambda event: self._expand())

		self.pdf_frame.columnconfigure(0, weight=3, minsize=100)
		self.pdf_frame.rowconfigure(0, weight=3, minsize=25)
		self.pdf_scroll.config(command=self.pdf_text.yview)
		self.pdf_text.configure(state="disabled")
		
		# Grid widgets
		self.collections_dropdown.grid(row=0, column=0, columnspan=2, sticky='ew')
		self.title_label.grid(row=0, column=2, columnspan=3, sticky='ew')
		
		self.source_type_label.grid(row=1, column=0, columnspan=2, sticky='ew')
		self.url_label.grid(row=1, column=2, columnspan=3, sticky='ew')
		
		self.source_label.grid(row=2, column=0, rowspan=1, columnspan=1, sticky='ew')
		self.star_label.grid(row=2, column=2, sticky='ew', pady=20)
		self.read_label.grid(row=2, column=3, columnspan=1, sticky='ew', pady=20)
		self.pdf_label.grid(row=2, column=4, rowspan=1, sticky='w', pady=20)

		self.canvas.grid(row=3, column=0, sticky='nsew')
		self.source_scroll.grid(row=3, column=1, sticky='ns')
		self.text_widget.grid(row=3, column=2, columnspan=3, sticky='nsew')
		self.text_scroll.grid(row=3, column=5, sticky='ns')



	def _display_collection(self, source_type='all'):

		# Ungrid current sources (if any)
		for source in self.frame.grid_slaves():
			source.grid_forget()

		# Grid new source labels
		self.selected_collection = zoresearch.sources_db._select_collection(self.app_data, self.collection_name, source_type)

		for i, source in enumerate(self.selected_collection):
			print(f'[{i+1}/{len(self.selected_collection)}] {source.title}')
			# source._create_label(self.frame)
			print()
			source.label.grid(row=i, column=0, sticky='nsew')
			source.label.bind('<Button-1>', lambda event, source=source: self._display_source(source))


	def _display_source(self, source):

		# Save data from previously selected source
		if self.text_widget.get("1.0",tk.END).strip():
			self._save_source()
		
		# Reset formatting of source label widgets
		for tile in self.selected_collection:
			tile.label.config(background='#E5E5E5', relief='flat', fg='black')
			
			if not tile.read:
				tile.label.config(font=('open sans', 10, 'bold'))
			else:
				tile.label.config(font=('open sans', 10))

			tile.label.bind('<Enter>', _hover)
			tile.label.bind('<Leave>', _leave)

		# Highlight selected source widget
		self.selected_source = source
		source.read = True
		source.label.unbind('<Enter>')
		source.label.unbind('<Leave>')
		source.label.config(background='#32ade6', fg='white', font=('open sans', 10))

		# Change GUI labels to match selected source
		self.title_label.config(text=source.title)
		self.title_label.bind("<Button-1>", lambda e: self._open_link(source.attachment))

		self.url_label.config(text=source.url)
		self.url_label.bind("<Button-1>", lambda e: self._open_link(source.url))

		self.text_widget.delete('1.0', tk.END)
		self.text_widget.insert('end', source.all_notes.strip())
		self.text_widget.config(font=('open sans', 10))

		self.read_label.config(text="Mark 'Unread'")
		self.read_label.bind("<Button-1>", self._mark_unread)

		if source.starred:
			self.star_label.config(text = u'\u2605')
		else:
			self.star_label.config(text = u'\u2606')
		self.star_label.bind("<Button-1>", self._star_source)

		if self.show_pdf == True:
			self._display_PDF()

	def _display_PDF(self):
		source = self.selected_source

		if source.attachment == None:
			self.title_label.configure(cursor='arrow')
			self.pdf_text.configure(state="normal")
			self.pdf_text.delete('1.0', tk.END)
			self.pdf_text.insert(tk.END, 'PDF unavailable')
			self.pdf_text.configure(state="disabled")

		else:
			self.title_label.configure(cursor='hand2')
			self.pdf_text.configure(state="normal")
			self.pdf_text.delete('1.0', tk.END)

			open_pdf = fitz.open(source.attachment)
			
			self.images = []
			
			for page in open_pdf:
				pix = page.get_pixmap()
				pix1 = fitz.Pixmap(pix, 0) if pix.alpha else pix
				img = pix1.tobytes("ppm")
				timg = PhotoImage(data=img)
				self.images.append(timg)
			
			for i, img in enumerate(self.images):
				self.pdf_text.image_create(tk.END, image=img)
				self.pdf_text.insert(tk.END, '\n\n')

			self.pdf_text.configure(state="disabled")


	def _expand(self):
		self.show_pdf = not self.show_pdf

		# To display PDF pane
		if self.show_pdf == True:
			self.pdf_label.configure(text='Hide PDF')

			# Configure and grid widgets
			self.pdf_frame.grid(row=3, column=6, sticky='nsew')	
			# self.pdf_frame.columnconfigure(1, minsize=1000)
			self.pdf_text.grid(row=0, column=0, sticky='nsew')			
			self.pdf_scroll.grid(row=0, column=1, sticky='ns')

			# Display PDF
			self._display_PDF()

		# To remove pdf pane
		else:
			self.pdf_label.configure(text='Show PDF')
			self.pdf_frame.grid_forget()

			# self.title_label.grid(row=0, column=2, columnspan=2, sticky='nsew')
			# self.url_label.grid(row=1, column=2, columnspan=2, sticky='nsew')
	

def _hover(event):
	'''Check widget foreground and change only if not green'''
	current_fg = event.widget.cget("fg")
	# print(f'Foreground color: {current_fg}')

	event.widget.config(background='#32ade6')
	if current_fg != '#006400':
		event.widget.config(foreground='white')


def _leave(event):
	current_fg = event.widget.cget("fg")
	event.widget.config(background='#E5E5E5')
	if current_fg != '#006400':
		event.widget.config(foreground='black')


def _main(app_data, collection_name, app_data_location):

	# Create GUI object
	gui = Note_gui(app_data, collection_name, app_data_location)



from pathlib import Path
import sqlite3
import json
import os
import shutil
import pandas as pd
import pdfkit
import numpy as np

from zoresearch import source as create_source


def _sql_query(zotero_location):
	'''Creates a copy of the Zotero DB and queries for source metadata. Returns two DB 
	which have to be merged
	'''
	zotero_copy = os.path.dirname(zotero_location) + '\\zotero_copy.sqlite'
	shutil.copyfile(zotero_location, zotero_copy)
	con = sqlite3.connect(zotero_copy)

	df_1 = pd.read_sql_query('''
			SELECT 
				*
			FROM itemDataValues
			INNER JOIN itemData ON 
				itemDataValues.valueID = itemData.valueID
			INNER JOIN collectionItems ON
				itemData.itemID = collectionItems.itemID
			INNER JOIN items ON 
				items.itemID = collectionItems.itemID
			INNER JOIN collections ON 
				collections.collectionID = collectionItems.collectionID
			WHERE
				(itemData.fieldID = 1
				OR itemData.fieldID = 8 
				OR itemData.fieldID = 13
				)
				AND collections.libraryID = 1
				'''
			, con,
			)

	df_2 = pd.read_sql_query('''
			SELECT
				itemAttachments.itemID as attachID,
				itemAttachments.parentItemID,
				itemAttachments.path,
				items.key as attachkey
			FROM 
				items
			INNER JOIN
				itemAttachments
			ON
				items.itemID = itemAttachments.itemID
			INNER JOIN 
				collectionItems
			ON 
				itemAttachments.parentItemID = collectionItems.itemID
			INNER JOIN 
				collections 
			ON 
				collections.collectionID = collectionItems.collectionID
			WHERE
				collections.libraryID = 1
			''',
		con,
		)

	con.close()
	os.remove(zotero_copy)
	return ([df_1, df_2])


# TASK: figure out a way to get rid of this
def _lower(collections):
	'''Lower case for all collections'''
	# print(collections)
	new_list = ['all']
	for collection in collections:
		new_list.append(collection.lower())
	return new_list


def _process_data(zotero_data_raw, zotero_folder):
	df_1 = zotero_data_raw[0]
	df_2 = zotero_data_raw[1]

	# Change all data types to strings
	df_1 = df_1.astype(str)
	df_2 = df_2.astype(str)

	# Keep necessary columns
	df_1.columns.values[22] = "key_2" # disambiguate 'key' column
	df_1 = df_1[['value', 'itemID', 'clientDateModified', 'key', 'collectionName', 'fieldID']]
	df_1 = df_1.iloc[:, [0, 1, 4, 6, 7, 8]] # get rid of dupulicate cols (not sure why cols get duplicated)

	# Unstack metadata by fieldID ('title', 'short_title', 'url')
	unstacked = df_1.groupby(['key', 'fieldID'])['value'].aggregate('first').unstack()
	unstacked = unstacked.reset_index()
	unstacked = unstacked.rename(columns={'1': 'title', '8': 'short_title', '13': 'url'})
	df_1 = df_1.merge(unstacked,on='key')
	df_1 = df_1[['key', 'itemID', 'title', 'url', 'short_title', 'collectionName', 'clientDateModified']]

	#Remove sources without attachments from attachments dataset and remove 'storage:' prefix
	df_2 = df_2[df_2['path']!='None'].reset_index()
	df_2['path'] = df_2['path'].str.slice(start=8)

	# Label attachments by filetype ('pdf', 'html') and keep only 1 per source (prioritize 'pdf')
	df_2['attach_num'] = df_2.groupby('parentItemID')['attachkey'].transform('nunique')
	df_2['pdf'] = df_2['path'].str.endswith('pdf')
	df_2['html'] = df_2['path'].str.endswith('html')
	df_2 = df_2.drop(df_2[(df_2['attach_num'] > 1) & (df_2['pdf'] == False) & (df_2['html'] == False)].index) # catch any sources w/o attachments
	
	# Repeat labeling and remove HTML sources if PDF is available
	df_2['attach_num'] = df_2.groupby('parentItemID')['attachkey'].transform('nunique')
	df_2 = df_2.drop(df_2[(df_2['attach_num'] > 1) & (df_2['pdf'] == False) & (df_2['html'] == True)].index)

	# Merge datasets
	df_merge = pd.merge(df_1, df_2, how='left', left_on=df_1['itemID'], right_on=df_2['parentItemID'])
	df_merge['attachment'] = zotero_folder + '\\storage\\' + df_merge['attachkey'] + '\\'
	df_merge = df_merge[['key', 'title',  'short_title', 'url', 'attachment', 'collectionName', 'clientDateModified']]
	df_merge = df_merge.rename(columns={"clientDateModified": "date_added"})
	df_merge = df_merge.drop_duplicates()
	collections = df_merge.groupby(['key'])['collectionName'].apply(lambda x: list(x.unique())).reset_index(name='collections')
	df_merge = df_merge.merge(collections, on='key')
	df_merge = df_merge.drop(['collectionName'], axis=1)
	df_merge.sort_values(by=['title'], inplace=True)
	df_merge.reset_index(inplace=True)
	df_merge = df_merge.groupby('key').first().reset_index()
	df_merge = df_merge.drop('index', axis=1)

	df_merge['collections'] = df_merge['collections'].apply(lambda x: _lower(x))
	zotero_data_processed = df_merge.to_dict('records')

	return zotero_data_processed


def _update_data(app_json_location, zotero_data):
	'''Adds and removes sources from local data set to match Zotero master'''

	# If no app data exists, just use Zotero data
	if not os.path.exists(app_json_location):
		print('No dataset exists. Creating from Zotero data')
		app_data = [create_source.Source(entry) for entry in zotero_data]

		return app_data

	# If app data exists, update sources to match Zotero data
	print('1. UPDATING SOURCES IN DATASET')
	with open(app_json_location, 'r') as f:
		app_data = json.load(f)

	# Add new sources from Zotero to app data
	for source in zotero_data:
		if source['key'] not in [source['key'] for source in app_data]:
			app_data.append(source)
			print(f'Adding {source["title"]}')

	# Remove sources deleted from Zotero from app data
	for source in app_data:
		if source['key'] not in [source['key'] for source in zotero_data]:
			app_data.remove(source)
			print(f'Removing {source["title"]}')

	# Update app data collection info to match Zotero 
	for app_source in app_data:
		for zot_source in zotero_data:
			if app_source['key'] == zot_source['key']:
				app_source['collections'] = zot_source['collections']
				break

	# Create Source object for each source
	app_data = [create_source.Source(entry) for entry in app_data]

	for i, source in enumerate(app_data):
		# print(f'[{i+1}/{len(app_data)}] {source.title}')
		# source = Source(source)
		source._get_attachment()
		source._get_annots()

	# Return updated dataset
	return app_data


def _select_collection(app_data, collection_name, source_type):
	selected_collection = [source for source in app_data if collection_name in source.collections]
	selected_collection = sorted(selected_collection, key=lambda source: source.date_added, reverse=True)

	if source_type == 'starred':
		selected_collection = [source for source in selected_collection if source.starred==True]

	if source_type == 'unread':
		selected_collection = [source for source in selected_collection if source.read==False]

	return selected_collection


